﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AsyncCalcPiWithBackgroundWorkerSample
{

    partial class AsyncCalcPiForm : Form
    {
        public AsyncCalcPiForm()
        {
            InitializeComponent();
        }

        void ShowProgressState(CalcPiUserState state)
        {
            try
            {
                // Display progress in UI
                this.resultsTextBox.Text = state.Pi;
                this.calcToolStripProgressBar.Maximum = state.TotalDigits;
                this.calcToolStripProgressBar.Value = state.DigitsSoFar;
            }
            catch (Exception e)
            {
                Console.WriteLine("{0} in ShowProgressState", e.Message);
                throw e;
            }
        }

        //delegate void ShowProgressDelegate(
        //  string pi, int totalDigits, int digitsSoFar);

        class CalcPiUserState
        {
            public readonly string Pi;
            public readonly int TotalDigits;
            public readonly int DigitsSoFar;

            public CalcPiUserState(string pi, int totalDigits, int digitsSoFar)
            {
                this.Pi = pi;
                this.TotalDigits = totalDigits;
                this.DigitsSoFar = digitsSoFar;
            }
        }

        void CalcPiWithProgress(int digits, IProgress<CalcPiUserState> progress)
        {
            StringBuilder pi = new StringBuilder("3", digits + 2);

            // Report initial progress
            progress.Report(new CalcPiUserState(pi.ToString(), digits, 0));

            if (digits > 0)
            {
                pi.Append(".");

                for (int i = 0; i < digits; i += 9)
                {
                    int nineDigits = NineDigitsOfPi.StartingAt(i + 1);
                    int digitCount = Math.Min(digits - i, 9);
                    string ds = string.Format("{0:D9}", nineDigits);
                    pi.Append(ds.Substring(0, digitCount));

                    // Report continuing progress
                    progress.Report(new CalcPiUserState(pi.ToString(), digits, digitCount + i));

                }
            }
        }

        void SetUIReady()
        {
            // Reset progress UI
            this.calcToolStripStatusLabel.Text = "Ready";
            this.calcToolStripProgressBar.Visible = false;
        }

        void SetUIBusy()
        {
            // Set calculating UI
            this.calcToolStripProgressBar.Visible = true;
            this.calcToolStripStatusLabel.Text = "Calculating...";
        }

        async void calcButton_Click(object sender, EventArgs e)
        {
            try
            {
                SetUIBusy();

                // Begin calculating pi asynchronously
                TimeSpan elapsed = await DoWork();

                // Show elapsed time
                MessageBox.Show("Elapsed: " + elapsed.ToString());

            }
            catch (Exception ex)
            {
                this.resultsTextBox.Text = ex.Message;
            }
            finally
            {
                SetUIReady();
            }
        }

        //Task piTask = null;
        async Task<TimeSpan> DoWork()
        {
            // Track start time
            DateTime start = DateTime.Now;

            if (checkBoxError.Checked)
            {
                throw new Exception("Oops");
            }
            //Any method that is an Action, has one parameter of the gerenic type and returns void.
            Action<CalcPiUserState> actionProgress = ShowProgressState;
            //The Action and the Progress must have the same generic type.
            IProgress<CalcPiUserState> progress = new Progress<CalcPiUserState>(actionProgress);
            await Task.Run(() => CalcPiWithProgress(
                  (int)this.decimalPlacesNumericUpDown.Value, progress)
                );

            // Return elapsed time
            DateTime end = DateTime.Now;
            TimeSpan elapsed = end - start;
            return elapsed;
        }

       
    }
}
